package com.flow.model.spreadsheet;

import lombok.Data;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.ss.util.RegionUtil;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;

import java.util.List;
import java.util.Map;
import java.util.Objects;

@Data
public class SpreadSheet {
    // 名称
    private String name;
    // 冻结
    private String freeze;
    // 自动过滤
    private Autofilter autofilter;
    // 列
    private Map<Integer, Col> cols;
    // 合并
    private List<String> merges;
    // 样式
    private List<SheetCellStyle> styles;
    // 校验
    private List<Object> validations;
    // 行
    private Map<Integer, SheetRow> rows;
    // 行数
    private Integer rowLen;
    // 列数
    private Integer colLen;

    public SXSSFWorkbook toExcel() {
        SXSSFWorkbook workbook = new SXSSFWorkbook();
        SXSSFSheet sheet = workbook.createSheet(this.name);
        sheet.setDefaultColumnWidth((100 - 5) / 7);
        sheet.setDefaultRowHeightInPoints(25 * 0.75f);
        // 行列
        for (Map.Entry<Integer, SheetRow> entry : this.rows.entrySet()) {
            Row row = sheet.createRow(entry.getKey());
            SheetRow sheetRow = entry.getValue();
            Integer height = sheetRow.getHeight();
            if (Objects.nonNull(height)) {
                row.setHeightInPoints(height);
            }
            Map<Integer, SheetCell> cells = sheetRow.getCells();
            for (Map.Entry<Integer, SheetCell> cellEntry : cells.entrySet()) {
                SheetCell poiSheetCell = cellEntry.getValue();
                Cell cell = row.createCell(cellEntry.getKey());
                cell.setCellValue(poiSheetCell.getText());
                Integer styleIndex = poiSheetCell.getStyle();
                if (Objects.nonNull(styleIndex)) {
                    SheetCellStyle sheetCellStyle = this.styles.get(styleIndex);
                    CellStyle cellStyle = sheetCellStyle.toStyle(workbook);
                    cell.setCellStyle(cellStyle);
                }
            }
        }
        // 合并区域
        for (String merge : this.merges) {
            CellRangeAddress cra = CellRangeAddress.valueOf(merge);
            sheet.addMergedRegion(cra);
            Cell cell = sheet.getRow(cra.getFirstRow()).getCell(cra.getFirstColumn());
            if (Objects.nonNull(cell)) {
                CellStyle cellStyle = cell.getCellStyle();
                // 左边边框
                BorderStyle borderLeft = cellStyle.getBorderLeft();
                short leftBorderColor = cellStyle.getLeftBorderColor();
                if (borderLeft != BorderStyle.NONE) {
                    RegionUtil.setBorderLeft(borderLeft, cra, sheet);
                    RegionUtil.setLeftBorderColor(leftBorderColor, cra, sheet);
                }
                // 右边边框
                BorderStyle borderRight = cellStyle.getBorderRight();
                short rightBorderColor = cellStyle.getRightBorderColor();
                if (borderRight != BorderStyle.NONE) {
                    RegionUtil.setBorderRight(borderRight, cra, sheet);
                    RegionUtil.setRightBorderColor(rightBorderColor, cra, sheet);
                }
                // 上边边框
                BorderStyle borderTop = cellStyle.getBorderTop();
                short topBorderColor = cellStyle.getTopBorderColor();
                if (borderTop != BorderStyle.NONE) {
                    RegionUtil.setBorderTop(borderTop, cra, sheet);
                    RegionUtil.setTopBorderColor(topBorderColor, cra, sheet);
                }
                // 下边边框
                BorderStyle borderBottom = cellStyle.getBorderBottom();
                short bottomBorderColor = cellStyle.getBottomBorderColor();
                if (borderBottom != BorderStyle.NONE) {
                    RegionUtil.setBorderBottom(borderBottom, cra, sheet);
                    RegionUtil.setBottomBorderColor(bottomBorderColor, cra, sheet);
                }
            }
        }
        return workbook;
    }

}
